<?php

namespace Goldenscarab\Modulus\Debusca\App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Response;
use Goldenscarab\Modulus\Debusca\App\Models\Path;
use Goldenscarab\Modulus\Debusca\App\Models\Folder;
use Goldenscarab\Modulus\Debusca\App\Models\File as ModelFile;
use Goldenscarab\Modulus\Debusca\App\Http\Services\Debusca\Facade\DebuscaService;



class FileController extends Controller
{

    public function __construct()
    {
        //ini_set('display_errors', false);
	    set_time_limit(0);
    }

    /**
     * Page principale
     *
     * @return View
     */
    public function index()
    {
        return view('debusca::app.file.index')->with([
            'title'    => "Debusca",
            'subtitle' => "Fichier",
        ]);
    }

    /**
     * Retourne le HTML de l'arborescence courante
     *
     * @param Request $request
     * @return View
     */
    public function tree(Request $request)
    {
        $path = new Path($request->get('path'));

        // Récupération des dossiers
        $folders = collect(File::directories($path->current))->sortBy(function($directory) {
            return strtolower($directory);
        })->values()->map(function($directory) use ($path) {
            $dirname = preg_replace('/^(\/.*\/)/', '', $directory);

            return new Folder($path->current, $dirname);
        });

        // Récupération de fichiers
        $files = collect(File::files($path->current, true))->sortBy(function($file) {
            return strtolower($file->getBaseName());
        })->values()->map(function($file) use ($path) {

            return new ModelFile($path->current, $file->getBaseName());
        });

        return view('debusca::app.file.partials._tree')->with([
            'folders' => $folders,
            'files'   => $files,
            'path'    => $path
        ]);
    }

    /**
     * Retourne le contenu d'un fichier
     *
     * @param Request $request
     * @return String
     */
    public function content(Request $request)
    {
        return File::get($request->get('pathname'));
    }

    /**
     * Download d'un fichier
     *
     * @param Request $request
     * @return Response
     */
    public function download(Request $request)
    {
        $pathname = $request->get('pathname');

        if (!File::exists($pathname)) {
            abort(404);
        }

        $file = File::get($pathname);
        $filetype = filetype($pathname);
        $filename = basename($pathname);

        $headers_response = [
            'Content-type'        => $filetype,
            'Content-Disposition' => 'inline; filename="' . $filename . '"',
        ];

        return Response::make($file, 200, $headers_response);
    }

    /**
     * Upload d'un fichier
     *
     * @param Request $request
     * @return Response
     */
    public function upload(Request $request)
    {
        $file = $request->file('file');
        $path = $request->get('path');

        $original_name = $file->getClientOriginalName();

        $filename = $this->makeUploadFilename($path, $original_name);

        $file->move($path, $filename);

        return Response::json(['success' => 'Fichier ajouté avec succès']);
    }

    private function makeUploadFilename($path, $filename)
    {
        $pathname = $path . '/' . $filename;

        if (File::exists($pathname)) {
            $this->makeUploadFilename($path, $filename . ' (1)');
        }

        return $filename;
    }

    public function find(Request $request)
    {
        $path = $request->get('path');
        $search = $request->get('search');

        $files = DebuscaService::find($path, $search)->map(function($file) {
            return new ModelFile($file->getPath(), $file->getBaseName());
        });

        return view('debusca::app.file.partials._tree')->with([
            'files'   => $files,
        ]);
    }

    public function grep(Request $request)
    {
        $path = $request->get('path');
        $search = $request->get('search');

        $finded = DebuscaService::grep($path, $search);

        $files = $finded->map(function($find) {
            $file = $find['file'];
            return new ModelFile($file->getPath(), $file->getBaseName());
        });

        $explorer = view('debusca::app.file.partials._tree')->with([
            'files'   => $files,
        ])->render();

        $editor = view('debusca::app.file.partials._grep')->with([
            'finded'   => $finded,
        ])->render();

        return response()->json([
            'explorer' => $explorer,
            'editor'   => $editor
        ]);
    }

}
