<script>
    class DebuscaBDD {
        constructor() {
            this.init();
        }

        init() {
            this.db_name = $('#field-database').val();
            this.initEditor();
            this.initListeners();
            this.refreshTablesList();
            this.loadSuggestion();
            this.getQueryString('create', function(query) {
                $('#create-table').attr('data-query', query);
            })
        }

        initEditor() {
            var _this = this;

            // Editeur
            var editor = ace.edit("editor");
            editor.setTheme("ace/theme/monokai");
            editor.session.setMode("ace/mode/sql");
            editor.setOption("enableEmmet", true);
            editor.getSession().setUseWrapMode(true);
            editor.setHighlightActiveLine(true);
            editor.setShowPrintMargin(false);
            editor.setFontSize("16px");
            var database = $('#field-database option:selected').attr('data-database');
            var savedValue = localStorage.getItem('debusca.bdd.editor.' + database);
            if (savedValue) {
                editor.setValue(savedValue);
            }
            editor.selection.clearSelection();

            editor.getSession().on('change', function(e) {
                var database = $('#field-database option:selected').attr('data-database');
                localStorage.setItem('debusca.bdd.editor.' + database, editor.getValue());
            });

            editor.commands.addCommand({
                name: 'validate',
                bindKey: {
                    win: 'Ctrl-Enter',
                    mac: 'Ctrl-Enter',
                    sender: 'editor|cli'
                },
                exec: function(env, args, request) {

                    var selection = editor.selection.getRange();

                    // Si pas de selection on selectionne la ligne
                    if (selection.start.column == selection.end.column) {
                        editor.selection.clearSelection();
                        editor.selection.selectLine();
                    }

                    var query = editor.getSelectedText();

                    if (query) {
                        _this.sendQuery(query);
                    }
                }
            });

            // Autocompletion
            this.langTools = ace.require("ace/ext/language_tools");
            editor.setOptions({
                enableBasicAutocompletion: true,
                enableLiveAutocompletion: true,
                enableSnippets: false
            });
            this.editor = editor;
            window.editor = editor;
        }

        initListeners() {

            var _this = this;

            // Au changement d'une base de données
            $(document).on('click', '#field-database', function() {
                _this.db_name = $(this).val();
                _this.refreshTablesList();
                _this.loadSuggestion();

                var savedValue = localStorage.getItem('debusca.bdd.editor.' + $(this).val());
                if (savedValue) {
                    _this.editor.setValue(savedValue);
                } else {
                    _this.editor.setValue('-- Appuyez sur Ctrl+Enter pour exécuter votre requête --');
                }
            });

            // Edition et Execution d'une requête
            $(document).on('click', '.edit-send-query', function(e) {
                e.stopPropagation();
                var query = $(this).attr('data-query');

                if (query) {
                    _this.addToEditor(query);
                    _this.sendQuery(query);
                }
            });

            // Edition d'une requête seule
            $(document).on('click', '.edit-query', function(e) {
                e.stopPropagation();
                var query = $(this).attr('data-query');

                if (query) {
                    _this.addToEditor(query);
                }
            });

            // Execution d'une requête seule
            $(document).on('click', '.send-query', function(e) {
                e.stopPropagation();
                var query = $(this).attr('data-query');

                if (query) {
                    _this.sendQuery(query);
                }
            });

            // Rafraichir la liste des tables
            $(document).on('click', '.refresh-tables', function() {
                _this.refreshTablesList(false);
            });

            // Click sur le bouton de recherche
            $(document).on('click', '#btn-search', function() {
                _this.refreshTablesList();
            });

            // Recherche d'une table
            $(document).on('keydown', '#field-search', function(e) {
                if (e.which === 13) {
                    _this.refreshTablesList();
                }
            });
        }

        getQueryString(query, callback) {
            $.ajax({
                url      : "{{ route('debusca.bdd.query.string') }}",
                type     : 'GET',
                data     : {'db_name': this.db_name, 'query': query},
                success  : function(result){
                    callback(result.query)
                },
                error    : function(result){
                    console.log(result)
                    var status = result.status;
                    var message = result.responseJSON.message
                    Toast.fire({
                        icon: 'error',
                        title: "Impossible de récupérer la requête " + query,
                        text: '[' + status + '] ' + message
                    })
                },
            });
        }

        refreshTablesList(cache = true) {
            $('#tables').addClass('is-loading').find('.list_').html('');
            var search = $('#field-search').val();

            $.ajax({
                url      : "{{ route('debusca.bdd.tables') }}",
                type     : 'GET',
                dataType : 'html',
                data     : {'db_name': this.db_name, 'cache': cache, 'search': search},
                success  : function(result){
                    $('#tables').removeClass('is-loading').find('.list').html(result);
                },
                error    : function(result){
                    console.log(result)
                    var status = result.status;
                    var message = result.responseJSON.message
                    Toast.fire({
                        icon: 'error',
                        title: "Impossible de récupérer la liste des tables",
                        text: '[' + status + '] ' + message
                    })
                },
            });
        }

        loadSuggestion() {
            var _this = this;

            $.ajax({
                url      : "{{ route('debusca.bdd.suggestions') }}",
                type     : 'GET',
                dataType : 'json',
                data     : {'db_name': this.db_name},
                success  : function(suggestions){

                    // Mise à jour de l'editeur
                    var myCompleter = {
                        identifierRegexps: [/[^\s|\.]+/],
                        getCompletions: function(editor, session, pos, prefix, callback) {
                            callback(
                                null,
                                suggestions.filter(entry => {
                                    return entry.includes(prefix);
                                }).map(entry => {
                                    return {
                                        value: entry
                                    };
                                })
                            );
                        }
                    }

                    _this.langTools.setCompleters([]);
                    _this.langTools.addCompleter(myCompleter);
                },
                error    : function(result){
                    console.log(result)
                    var status = result.status;
                    var message = result.responseJSON.message
                    Toast.fire({
                        icon: 'error',
                        title: "Impossible de charger les suggestions",
                        text: '[' + status + '] ' + message
                    })
                },
            });

        }

        sendQuery(query) {
            $('#query-result').addClass('is-loading').find('.list').html('');

            $.ajax({
                url      : "{{ route('debusca.bdd.query') }}",
                headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                type     : 'POST',
                data     : {'db_name': this.db_name, 'query': query },
                success  : function(html){
                    $('#query-result').removeClass('is-loading')
                    $('#query-result').find('.list').html(html);
                },
                error    : function(result){
                    $('#query-result').removeClass('is-loading');
                    console.log(result);
                    var status = result.status;
                    var message = result.responseJSON.message

                    var $title = $('<h4>').addClass('alert-heading').text(status);
                    var $text  = $('<p>').text(message);
                    var $alert = $('<div>').addClass('alert alert-danger').attr('role', 'alert');
                    $alert.append($title);
                    $alert.append($text);

                    $('#query-result').find('.list').html($alert);
                },
            });
        }

        jsonStatusToHtml(json) {
            var text = '';

            if (json.status != null) {
                var action = json.status.type.replace(/e$/, '') + 'ed';
                text = json.status.lines + ' row' + (json.status.lines > 1 ? 's' : '') + ' ' + action + ' in ' + json.status.elapsed_ms + ' ms';
            }

            return text;
        }

        addToEditor(value) {
            var current = this.editor.getValue();
            this.editor.setValue(current + "\n\n" + value);
            this.editor.execCommand("gotolineend");
        }
    }
</script>
